/*
   filegive - Easy sending of files
   Copyright (C) 2013  Lluís Batlle i Rossell

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU Affero General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Affero General Public License for more details.

   You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package main

import (
	"io"
	"time"
)

type RateLimit struct {
	ticker *time.Ticker
	rate   int
	stored int
}

func NewRateLimit(persec int) *RateLimit {
	rl := &RateLimit{}
	rl.rate = persec
	rl.ticker = time.NewTicker(time.Second)

	return rl
}

func (rl *RateLimit) Want(l int) int {
	if l > rl.rate {
		l = rl.rate
	}
	rl.stored -= l
	for rl.stored < 0 {
		<-rl.ticker.C
		rl.stored += rl.rate
	}
	return l
}

func (rl *RateLimit) Stop() {
	rl.ticker.Stop()
}

type LimitedRateReader struct {
	r io.Reader
}

func (lrr LimitedRateReader) Read(p []byte) (n int, err error) {
	if rateLimit != nil {
		l := rateLimit.Want(len(p))
		p = p[:l]
	}
	n, err = lrr.r.Read(p)
	return
}
